/****************************************************************************
 *   $Id:: adc.c 4785 2010-09-03 22:39:27Z nxp21346                         $
 *   Project: NXP LPC11xx ADC example
 *
 *   Description:
 *     This file contains ADC code example which include ADC 
 *     initialization, ADC interrupt handler, and APIs for ADC
 *     reading.
 *
 ****************************************************************************/
//#define	__SWD_DISABLED
#ifndef CONFIG_ENABLE_DRIVER_ADC
#define CONFIG_ENABLE_DRIVER_ADC 1
#endif

//#include "driver_config.h"
#if CONFIG_ENABLE_DRIVER_ADC==1
#include "adc.h"

volatile uint32_t ADCValue[ADC_NUM];

#if CONFIG_ADC_DEFAULT_ADC_IRQHANDLER==1
volatile uint32_t ADCIntDone = 0;
#endif
volatile uint32_t OverRunCounter = 0;

#if BURST_MODE
volatile uint32_t channel_flag = 0; 
#endif

#if CONFIG_ADC_DEFAULT_ADC_IRQHANDLER==1
/******************************************************************************
** Function name:		ADC_IRQHandler
**
** Descriptions:		ADC interrupt handler
**
** parameters:			None
** Returned value:		None
** 
******************************************************************************/
void ADC_IRQHandler (void) 
{
  uint32_t regVal, i;

  regVal = LPC_ADC->STAT;		/* Read ADC will clear the interrupt */
  if ( regVal & 0x0000FF00 )	/* check OVERRUN error first */
  {
	OverRunCounter++;
	for ( i = 0; i < ADC_NUM; i++ )
	{
	  regVal = (regVal & 0x0000FF00) >> 0x08;
	  /* if overrun, just read ADDR to clear */
	  /* regVal variable has been reused. */
	  if ( regVal & (0x1 << i) )
	  {
		regVal = LPC_ADC->DR[i];
	  }
	}
	LPC_ADC->CR &= 0xF8FFFFFF;	/* stop ADC now */
	ADCIntDone = 1;
	return;	
  }
    
  if ( regVal & ADC_ADINT )
  {
	for ( i = 0; i < ADC_NUM; i++ )
	{
	  if ( (regVal&0xFF) & (0x1 << i) )
	  {
		ADCValue[i] = ( LPC_ADC->DR[i] >> 6 ) & 0x3FF;
	  }
	}
#if CONFIG_ADC_ENABLE_BURST_MODE==1
	channel_flag |= (regVal & 0xFF);
	if ( (channel_flag & 0xFF) == 0xFF )
	{
	  /* All the bits in have been set, it indicates all the ADC 
	  channels have been converted. */
	  LPC_ADC->CR &= 0xF8FFFFFF;	/* stop ADC now */
	  channel_flag = 0; 
	  ADCIntDone = 1;
	}
#else
	LPC_ADC->CR &= 0xF8FFFFFF;	/* stop ADC now */ 
	ADCIntDone = 1;
#endif
  }
  return;
}
#endif
/*****************************************************************************
** Function name:		ADCInit
**
** Descriptions:		initialize ADC channel
**
** parameters:			ADC clock rate
** Returned value:		None
** 
*****************************************************************************/
void ADCInit( uint32_t ADC_Clk )
{
  uint32_t i;

  /* Disable Power down bit to the ADC block. */  
  LPC_SYSCON->PDRUNCFG &= ~(0x1<<4);

  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<13); /* Enable AHB clock to the ADC. */
  LPC_SYSCON->SYSAHBCLKCTRL |= (1<<16);     // Enable IOCON Clock

  for ( i = 0; i < ADC_NUM; i++ )
  {
	ADCValue[i] = 0x0;
  }
  LPC_IOCON->R_PIO0_11 &= ~0x8F; /*  ADC I/O config */
  LPC_IOCON->R_PIO1_0  |= 0x02;  /* ADC IN1 */
  LPC_IOCON->R_PIO1_1  &= ~0x8F;	
  LPC_IOCON->R_PIO1_1  |= 0x02;  /* ADC IN2 */
  LPC_IOCON->R_PIO1_2 &= ~0x8F;	
  LPC_IOCON->R_PIO1_2 |= 0x02; /* ADC IN3 */
#ifdef __SWD_DISABLED          /*Be carefull!*/
  LPC_IOCON->SWDIO_PIO1_3   &= ~0x8F;	
  LPC_IOCON->SWDIO_PIO1_3   |= 0x02;  /* ADC IN4 */
  LPC_IOCON->SWDIO_PIO1_3    = 0x02;	// Select AD4 pin function
#endif
//  LPC_IOCON->R_PIO0_11   = 0x02;	// Select AD0 pin function
  LPC_IOCON->R_PIO1_0    = 0x02;	// Select AD1 pin function
  LPC_IOCON->R_PIO1_1    = 0x02;	// Select AD2 pin function
  LPC_IOCON->R_PIO1_2    = 0x02;	// Select AD3 pin function  
  LPC_IOCON->PIO1_4    = 0x01;	// Select AD5 pin function

    LPC_ADC->CR |= (0x01<<1)|                     /* bit7:bit0   default channel:1 */
          (((SystemCoreClock/LPC_SYSCON->SYSAHBCLKDIV)/ADC_Clk-1)<<8 )|     /* bit15:bit8 Set ADC Clock 48/(23+1)*/
          (1<<16 )| 		          /* bit16 hardware scan*/
          (0<<17 )|  		          /* bit19:bit17 10 bit mode*/
    	  (0<<24 );                   /* bit26:bit24 in hardware mode,this bit should set to 0 */
    LPC_SYSCON->SYSAHBCLKCTRL &= ~(1<<16);    // Close IOCON Clock
  return;
}

/*****************************************************************************
** Function name:		ADCRead
**
** Descriptions:		Read ADC channel
**
** parameters:			Channel number
** Returned value:		Value read, if interrupt driven, return channel #
** 
*****************************************************************************/
#include "systick_delay.h"
uint32_t ADCRead( uint8_t channelNum )
{
  /* channel number is 0 through 7 */
  if ( channelNum > ADC_NUM )
  {
	channelNum = 1;		/* reset channel number to 1 */
  }
//    LPC_ADC->CR &= 0xFFFFFF00;
    LPC_ADC->CR =/* (1 << 24) | */LPC_ADC->CR|(0x01 << channelNum); //	change AD sample channel.
    while (!(LPC_ADC->STAT &(0x01<<channelNum))); //wait until STAT DONE bit != 1.
    return (( LPC_ADC->DR[channelNum] >> 6 ) & 0x3FF);
}

/*****************************************************************************
** Function name:		ADC0BurstRead
**
** Descriptions:		Use burst mode to convert multiple channels once.
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void ADCBurstRead( void )
{
  if ( LPC_ADC->CR & (0x7<<24) )
  {
	LPC_ADC->CR &= ~(0x7<<24);
  }
  /* Read all channels, 0 through 7. Be careful that if the ADCx pins is shared
  with SWD CLK or SWD IO. */
  LPC_ADC->CR |= (0xFF);
  LPC_ADC->CR |= (0x1<<16);		/* Set burst mode and start A/D convert */
  return;						/* the ADC reading is done inside the 
								handler, return 0. */
}
#endif

/*********************************************************************************
**                            End Of File
*********************************************************************************/
